<!DOCTYPE html>
<html>
{% include 'head.html' %}
<body>
    <div id="centered-container">
        {% include 'header.html' %}
        {% include 'sign-out.html' %}
        <div id="start-text">
            <p>
                You're almost ready to start using Sigmund, but you still need a subscription!
            </p>
            </p>
                <h2>What do you get?</h2>
                Unlimited access to an AI research assistant powered by the most powerful AI technologies currently available.
            <p>
                <h2> How does it work?</h2>
                A subscription to Sigmund AI is only €9 per month.
                Payment is handled securely through Stripe.
                You can (easily) cancel your subscription on a monthly basis.
            </p>
            <p>
            <a href="/subscribe/create-checkout-session" class="link-button">
                <i class="fab fa-cc-stripe"></i> Subscribe now!
            </a>
            </p>
            <p>
            If you need a custom subscription, such as an invoiced subscription
            for multiple individuals or a longer period, contact
            <a href="mailto:professional@cogsci.nl">professional@cogsci.nl</a>.
            </p>
            <p><a href="/terms">Read our terms and conditions</a></p>
        </div>
        {% include 'footer.html' %}
    </div>
</body>
</html>
