<div class="message message-ai">
    <p>
        Nice to meet you! I'm Sigmund, your friendly AI assistant. How can I help you?
    </p>
</div>

<div class="message message-user">Hi there Sigmund! What makes you unique?</div>

<div class="message message-ai">
    <p>
        I'm <b>the perfect scientific sidekick</b>, especially if you're a cognitive neuroscientist, psychologist, or data scientist!
    </p>
    <ul>
        <li>
            <b>Data science</b>: I can connect to <a href="https://github.com/open-cogsci/jupyter-extension-sigmund" target="_blank">JupyterLab, Notebook, Spyder</a>, and <a href="https://github.com/open-cogsci/sigmund-analyst" target="_blank">Sigmund Analyst</a>
        </li>
        <li>
            <b>Experiment building</b>: I can connect to <a href="https://osdoc.cogsci.nl/4.1/tutorials/beginner-sigmund/" target="_blank">OpenSesame</a>
        </li>
        <li>
            <b>Expert knowledge</b>: I know scientific Python libraries and OpenSesame really well
        </li>
        <li>
            <b>Literature search</b>: I can search Google Scholar for research papers
        </li>
    </ul>
    <p>
        &#x1F469;&#x1F3FE;&#x200D;&#x1F4BB; &#x1F3D7;&#xFE0F; &#x1F9E0; &#x1F4DA;
    </p>
</div>

<div class="message message-user">I care about privacy. Who can read our conversations?</div>

<div class="message message-ai">
    <p>
        <b>Everything is encrypted.</b> No one—not even my admins—can read along. &#x1F510; &#x2764;&#xFE0F;&#x200D;&#x1F525;
    </p>
</div>

<div class="message message-user">Sounds awesome! &#x1F31E; How do I get started?</div>

<div class="message message-ai">
    <p>
        First, <a onclick="app.__vue__.toggleMenu()" href="#">sign in</a> with your Google account or your account from forum.cogsci.nl. Once signed in, you'll need to subscribe (if you're not subscribed already!)
    </p>
    <ul>
        <li>
            A subscription is €9/month—much cheaper than other premium chatbots
        </li>
        <li>
            A variety of international payment methods are available—all easy and secure
        </li>
        <li>
            Cancel anytime
        </li>
        <li>
            If you need a custom subscription, such as an invoiced subscription for multiple individuals or a longer period, contact <a href="mailto:professional@cogsci.nl">professional@cogsci.nl</a>
        </li>
    </ul>
    <p>
        I hope to talk to you soon! &#x1F680; &#x1F496;
    </p>
</div>